<?php

namespace Drupal\required_by_role\Plugin\Required;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\required_api\Attribute\Required;
use Drupal\required_api\Plugin\Required\RequiredBase;
use Drupal\user\Entity\Role;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Required by role plugin.
 */
#[Required(
  id: self::ID,
  label: new TranslatableMarkup('Required by role'),
  description: new TranslatableMarkup('Required based on current user roles.'),
)]
class RequiredByRole extends RequiredBase {

  use StringTranslationTrait;

  public const ID = 'required_by_role';

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->currentUser = $container->get('current_user');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function isRequired(FieldDefinitionInterface $field, ContentEntityInterface $entity): bool {
    $available_roles = $this->currentUser->getRoles();
    $field_roles = $field->getThirdPartySetting('required_api', 'required_plugin_options', []);

    return $this->getMatches($available_roles, $field_roles);
  }

  /**
   * Helper method to test if the role exists into the allowed ones.
   *
   * @param array $user_roles
   *   Roles belonging to the user.
   * @param array $required_roles
   *   Roles that are required for this field.
   *
   * @return bool
   *   Whether the user have a required role or not.
   */
  public function getMatches(array $user_roles, array $required_roles): bool {
    $match = array_intersect($user_roles, $required_roles);

    return !empty($match);
  }

  /**
   * {@inheritdoc}
   */
  public function requiredFormElement(FieldDefinitionInterface $field): array {
    $default_value = $field->getThirdPartySetting('required_api', 'required_plugin_options') ?: [];
    $default_value = array_combine($default_value, array_fill(0, count($default_value), 1));

    $options = array_map(
      function ($role) {
        return ['name' => $role->label()];
      },
      Role::loadMultiple()
    );
    unset($options[AccountInterface::AUTHENTICATED_ROLE]);

    return [
      '#type' => 'tableselect',
      '#header' => [
        'name' => ['data' => $this->t('Role')],
      ],
      '#options' => $options,
      '#default_value' => $default_value,
      '#js_select' => TRUE,
      '#multiple' => TRUE,
      '#empty' => $this->t('No roles available.'),
      '#attributes' => [
        'class' => ['tableselect-required-by-role'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitFieldConfigForm(array &$form, FormStateInterface $form_state): void {
    $parents = ['third_party_settings', 'required_api', 'required_plugin_options'];
    $options = $form_state->getValue($parents);
    $form_state->setValue($parents, array_keys(array_filter($options)));
  }

}
